/*NeatUpload - an HttpModule and User Controls for uploading large files
Copyright (C) 2005  Dean Brettle

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

clientRefreshScript updated for Javascript Postback Module
released under GNU Lesser General Public License
Copyright (C) 2005  Stefano Straus (tustena.sf.net)
*/

using System;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using Brettle.Web.NeatUpload;
using Digita.Tustena.Base;

namespace Digita.Tustena.NeatUpload
{
	public class Progress : Page
	{

		protected HtmlGenericControl barDiv;
		protected HtmlGenericControl statusDiv;
		protected HtmlGenericControl inProgressSpan;
		protected HtmlGenericControl remainingTimeSpan;
		protected HtmlGenericControl completedSpan;
		protected HtmlGenericControl cancelledSpan;
		protected HtmlAnchor refreshLink;
		protected HtmlAnchor stopRefreshLink;
		protected HtmlAnchor cancelLink;
		protected HtmlImage refreshImage;
		protected HtmlImage stopRefreshImage;
		protected HtmlImage cancelImage;
		protected System.Web.UI.HtmlControls.HtmlGenericControl label;


		private string clientRefreshScript = @"<script id=""jsProgress""></script><script language=""javascript"">
function loadScript (url) {
  if (document.layers){
    window.location.href = url;
  }else if (document.getElementById) {
	if (navigator.appVersion.indexOf(""Mac"")!=-1){
	obj=document.createElement('div');
	obj.innerHTML='<sc' + 'ript type=\'text/javascript\' src=\'' + url + '\'><\/sc' + 'ript>';
	document.body.appendChild(obj);
	}else{
    var script = document.createElement('script');
    script.defer = true;
    script.src = url;
    document.getElementsByTagName('head')[0].appendChild(script);
    }
  }
}

function Progress()
{
	loadScript('/neatupload/jsProgress.aspx?'+document.location.search.substring(1)+'&' + Math.random());
}

function ProcessUpload()
{
if ( typeof( uploadprogress ) != ""undefined"" )
document.getElementById(""barDiv"").style.width=uploadprogress;
if ( typeof( uploadremainingtime ) != ""undefined"" )
document.getElementById(""remainingTimeSpan"").innerHTML=uploadremainingtime;
if ( typeof( uploadstatus ) != ""undefined"" ){
if(uploadstatus==""inprogress"")
	NeatUploadReloadTimeoutId = window.setTimeout(Progress, 1000);
else
	NeatUploadReloadTimeoutId = window.setTimeout(NeatUploadRefresh, 1000);
}
}

function NeatUploadRefresh()
{
	window.location.replace('REFRESHURL');
}

function NeatUploadGetMainWindow()
{
	var mainWindow;
	if (window.opener)
		mainWindow = window.opener;
	else
		mainWindow = window.parent;
	return mainWindow;
}

function NeatUploadCancel()
{
	var mainWindow = NeatUploadGetMainWindow();
	if (mainWindow.stop)
		mainWindow.stop();
	else if (mainWindow.document.execCommand)
		mainWindow.document.execCommand('Stop');
}

NeatUploadReloadTimeoutId = window.setTimeout(Progress, 1000);

NeatUploadMainWindow = NeatUploadGetMainWindow();
if (NeatUploadMainWindow.stop == null && NeatUploadMainWindow.document.execCommand == null)
{
	NeatUploadLinkNode = document.getElementById('cancelLink');
	if (NeatUploadLinkNode)
		NeatUploadLinkNode.parentNode.removeChild(NeatUploadLinkNode);
}
</script>";

		protected override void OnInit(EventArgs e)
		{
			InitializeComponent();
			base.OnInit(e);
		}

		private void InitializeComponent()
		{
			this.PreRender += new System.EventHandler(this.Page_PreRender);

		}

		private void Page_PreRender(object sender, EventArgs e)
		{
			string refreshUrl = Request.Url.AbsoluteUri;
			int ampIndex = refreshUrl.IndexOf("&");
			if (ampIndex != -1)
			{
				refreshUrl = refreshUrl.Substring(0, ampIndex);
			}
			cancelLink.Visible = false;
			stopRefreshLink.Visible = false;
			refreshLink.Visible = true;
			refreshLink.HRef = refreshUrl + "&refresher=server";
			RegisterStartupScript("scrNeatUpload", "<script language=\"javascript\">"
				+ "NeatUploadLinkNode = document.getElementById('" + refreshLink.ClientID + "'); if (NeatUploadLinkNode) NeatUploadLinkNode.parentNode.removeChild(NeatUploadLinkNode);"
				+ "</script>");

			string postBackID = Request.Params["postBackID"];
			UploadContext uploadContext = UploadContext.FindByID(postBackID);

			if (uploadContext != null && Request.Params["cancelled"] == "true")
			{
				uploadContext.Status = UploadStatus.Cancelled;
			}

			UpdateProgressBar(uploadContext);

			string prevStatus = Request.Params["prevStatus"];
			if (prevStatus == null)
			{
				prevStatus = UploadStatus.Unknown.ToString();
			}
			string curStatus = UploadStatus.Unknown.ToString();
			if (uploadContext != null)
			{
				curStatus = uploadContext.Status.ToString();
			}

			if (curStatus == prevStatus
				&& (curStatus == UploadStatus.Completed.ToString() || curStatus == UploadStatus.Cancelled.ToString()))
			{
				RegisterStartupScript("scrNeatUploadClose", "<script language=\"javascript\">window.close();</script>");
			}
			else if (Request.Params["refresh"] != "false" && Request.Params["refresher"] != null)
			{
				refreshLink.Visible = false;

				refreshUrl += "&prevStatus=" + curStatus;

				Refresh(refreshUrl);
			}
		}

		private void UpdateProgressBar(UploadContext uploadContext)
		{
			inProgressSpan.Visible = false;
			completedSpan.Visible = false;
			cancelledSpan.Visible = false;

			if (uploadContext == null || uploadContext.Status == UploadStatus.Unknown)
			{
				string lastPostBackID = Request.Params["lastPostBackID"];
				if (StaticFunctions.IsNotBlank(lastPostBackID) && Request.Params["refresher"] == null)
				{
					uploadContext = UploadContext.FindByID(lastPostBackID);
					if (uploadContext.NumUploadedFiles == 0)
					{
						uploadContext = null;
					}
				}
			}

			if (uploadContext == null)
			{
				barDiv.Style["width"] = "0";
			}
			else
			{
				barDiv.Style["width"] = uploadContext.PercentComplete + "%";
				if (uploadContext.Status == UploadStatus.Cancelled)
				{
					cancelledSpan.Visible = true;
				}
				else if (uploadContext.Status == UploadStatus.InProgress)
				{
					TimeSpan tr = uploadContext.TimeRemaining;
					remainingTimeSpan.InnerHtml = String.Format("{0:00}:{1:00}", (int)Math.Floor(tr.TotalMinutes), tr.Seconds);
					inProgressSpan.Visible = true;
				}
				else if (uploadContext.Status == UploadStatus.Completed)
				{
					completedSpan.Visible = true;
				}
			}
		}

		private void Refresh(string refreshUrl)
		{
			if (Request.Params["refresher"] == "client")
			{
				RefreshWithClientScript(refreshUrl);
			}
			else
			{
				RefreshWithServerHeader(refreshUrl);
			}
		}

		private void RefreshWithClientScript(string refreshUrl)
		{
			refreshUrl += "&refresher=client";
			cancelLink.Visible = true;
			cancelLink.HRef = refreshUrl + "&cancelled=true";
			cancelLink.Attributes["onclick"] = "javascript: if (NeatUploadReloadTimeoutId != null) window.clearTimeout(NeatUploadReloadTimeoutId); NeatUploadCancel();";
			RegisterStartupScript("scrNeatUploadRefresh", clientRefreshScript.Replace("REFRESHURL", refreshUrl));
		}

		private void RefreshWithServerHeader(string refreshUrl)
		{
			refreshUrl += "&refresher=server";
			Response.AddHeader("Refresh", "1; URL=" + refreshUrl);
			stopRefreshLink.Visible = true;
			stopRefreshLink.HRef = refreshUrl + "&refresh=false";
		}
	}
}
