/// <license>TUSTENA PUBLIC LICENSE v1.0</license>
/// <copyright>
/// Portions Copyright (c) 2003-2005 Digita S.r.l. All Rights Reserved.
///
/// Tustena CRM is a trademark of:
/// Digita S.r.l.
/// Viale Enrico Fermi 14/z
/// 31011 Asolo (Italy)
/// Tel. +39-0423-951251
/// Mail. info@digita.it
///
/// This file contains Original Code and/or Modifications of Original Code
/// as defined in and that are subject to the Tustena Public Source License
/// Version 1.0 (the 'License'). You may not use this file except in
/// compliance with the License. Please obtain a copy of the License at
/// http://www.tustena.com/TPL/ and read it before using this
// file.
///
/// The Original Code and all software distributed under the License are
/// distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
/// EXPRESS OR IMPLIED, AND DIGITA S.R.L. HEREBY DISCLAIMS ALL SUCH WARRANTIES,
/// INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
/// FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
/// Please see the License for the specific language governing rights and
/// limitations under the License.
///
/// YOU MAY NOT REMOVE OR ALTER THIS COPYRIGHT NOTICE!
/// </copyright>

using System;
using System.Data;
using System.Web;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.DomValidators;
using Digita.Tustena.Database;

namespace Digita.Tustena.WorkingCRM
{

	public class ActivityInsert
	{
		public ActivityInsert()
		{
		}

		public void InsertReminder(int id, DateTime data, UserConfig uC, string ownerID, int advanceremind)
		{
			using (DigiDapter dg = new DigiDapter())
			{
				dg.Add("OwnerID", ownerID, 'I');
				dg.Add("K_ID", id, 'I');
				dg.Add("TableName", "CRM_WorkActivity", 'I');

				dg.Add("ReminderDate", data);
				try
				{
					dg.Add("Note", HttpContext.Current.Session["Reminder_RemNote"].ToString());
					HttpContext.Current.Session.Remove("Reminder_RemNote");
				}
				catch
				{
					dg.Add("Note", String.Empty);
				}

				dg.Add("AdvanceRemind", advanceremind);

				dg.Execute("CRM_Reminder", "K_ID =" + id + " and TableName='CRM_WorkActivity' and OwnerID=" + uC.UserId);
			}
		}

		public void InsertActivity(string a, string visitID, string uid, string contactID, string companyID, string leadID, string subject, string description, DateTime data, UserConfig uC, int todo)
		{
			InsertActivity(a, visitID, uid, contactID, companyID, leadID, subject, description, data, uC, todo, true, 0,0);
		}

		public void InsertActivity(string a, string visitID, string uid, string contactID, string companyID, string leadID, string subject, string description, DateTime data, UserConfig uC)
		{
			InsertActivity(a, visitID, uid, contactID, companyID, leadID, subject, description, data, uC, 0, true, 0, 0);
		}

		public void InsertActivity(string a, string visitID, string uid, string contactID, string companyID, string leadID, string subject, string description, DateTime data, UserConfig uC, int todo,int daysallarm)
		{
			InsertActivity(a, visitID, uid, contactID, companyID, leadID, subject, description, data, uC, 0, true, 0, daysallarm);
		}

		public void InsertActivity(string a, string visitID, string uid, string contactID, string companyID, string leadID, string subject, string description, DateTime data, UserConfig uC, int todo, bool inout, long docid, int daysallarm)
		{
			using (DigiDapter dg = new DigiDapter())
			{
				dg.Add("Type", a, 'I');
				dg.Add("CreatedDate", DateTime.UtcNow, 'I');
				dg.Add("CreatedByID", uC.UserId, 'I');
				if (visitID.Length > 0) dg.Add("VisitID", visitID, 'I');

				dg.Add("OwnerID", uid.ToString());

				dg.Add("Subject", subject);

				dg.Add("Description", description);
				if (companyID.Length > 0)
					dg.Add("CompanyID", companyID);
				if (contactID.Length > 0)
					dg.Add("ReferrerID", contactID);
				if (leadID.Length > 0)
					dg.Add("LeadID", leadID);

				dg.Add("Groups", "|" + uC.UserGroupId.ToString() + "|");
				dg.Add("LastModifiedDate", DateTime.UtcNow);
				dg.Add("LastModifiedByID", uC.UserId);
				dg.Add("InOut", inout);
				dg.Add("ToDo", todo);

				dg.Add("ActivityDate", data);
				dg.Add("ParentID", 0);
				dg.Add("State", 0);
				dg.Add("Priority", 0);

				dg.Add("ToBill", false);
				dg.Add("Commercial", false);
				dg.Add("Technical", false);

				dg.Add("Allarm", data);
				dg.Add("DaysAllarm", daysallarm);

				dg.Add("Duration", 0);
				if (docid > 0)
					dg.Add("DocID", docid);

				if (visitID.Length > 0)
				{
					object newId = dg.Execute("CRM_WorkActivity", "VisitID=" + visitID, DigiDapter.Identities.Identity);
					if (dg.RecordInserted)
						InsertReminder(Convert.ToInt32(newId), data, uC, uid.ToString(), daysallarm);
					else
					{
						int OldID = int.Parse(DatabaseConnection.SqlScalar("select id from CRM_WorkActivity where VisitID=" + visitID));
						InsertReminder(OldID, data, uC, uid.ToString(), daysallarm);
					}
				}
				else
				{
					dg.Execute("CRM_WorkActivity");
				}
			}
		}
	}


	public class WorkingCRM : G
	{

		protected virtual void FillDropDown(string a)
		{
			DropDownList dp = (DropDownList) Page.FindControl("DropDownListStatus");
			dp.Items.Clear();
			dp.Items.Add(new ListItem(rm.GetString("Wortxt0"), "0"));
			dp.Items.Add(new ListItem(rm.GetString("Wortxt1"), "1"));
			dp.Items.Add(new ListItem(rm.GetString("Wortxt2"), "2"));
			dp.Items[0].Selected = true;
			dp = (DropDownList) Page.FindControl("DropDownListPriority");
			dp.Items.Clear();
			dp.Items.Add(new ListItem(rm.GetString("Wortxt3"), "0"));
			dp.Items.Add(new ListItem(rm.GetString("Wortxt4"), "1"));
			dp.Items.Add(new ListItem(rm.GetString("Wortxt5"), "2"));
			dp.Items[0].Selected = true;
			dp = (DropDownList) Page.FindControl("DropDownListClassification");
			dp.Items.Clear();
			DataSet wc = DatabaseConnection.CreateDataset("select ID,Description from CRMWorkingClassification where type=" + a + " and lang='" + UC.Culture.Substring(0, 2) + "' order by DropPosition");
			if (wc.Tables[0].Rows.Count > 0)
			{
				dp.DataTextField = "Description";
				dp.DataValueField = "ID";
				dp.DataSource = wc;
				dp.DataBind();
				dp.Items[0].Selected = true;
			}

			dp = (DropDownList) Page.FindControl("DropDownListPreAlarm");
			dp.Items.Clear();
			for (int i = 0; i < 6; i++)
			{
				dp.Items.Add(new ListItem(i.ToString() + " " + rm.GetString("Acttxt124"), i.ToString()));
			}
			dp.Items[0].Selected = true;


		}


		private void ModifyReminderActivity(int id, string data)
		{
			using (DigiDapter dg = new DigiDapter())
			{
				dg.Add("OwnerID", ((TextBox) Page.FindControl("TextboxOwnerID")).Text, 'I');
				dg.Add("K_ID", id, 'I');
				dg.Add("TableName", "CRM_WorkActivity", 'I');
				dg.Add("ReminderDate", UC.LTZ.ToUniversalTime(Convert.ToDateTime(data, UC.myDTFI)));
				dg.Add("Note", ((TextBox) Page.FindControl("TextBoxSubject")).Text);
				dg.Add("AdvanceRemind", (((DropDownList) Page.FindControl("DropDownListPreAlarm")).SelectedValue.Length > 0) ? ((DropDownList) Page.FindControl("DropDownListPreAlarm")).SelectedValue : "0");

				dg.Execute("CRM_Reminder", "K_ID =" + id + " and TableName='CRM_WorkActivity' and OwnerID=" + int.Parse(((TextBox) Page.FindControl("TextboxOwnerID")).Text));
			}
		}


		protected virtual string InsModActivity(string a, int id)
		{
		{
			bool PageValid = true;
			string errorCode = String.Empty;
			if (((RadioButtonList) Page.FindControl("Activity_ToDo")).Items[1].Selected && a == "6")
			{
				if (!((RequiredDomValidator) Page.FindControl("StartHourValidator")).IsValid ||
					!((RequiredDomValidator) Page.FindControl("EndHourValidator")).IsValid)
				{
					PageValid = false;
					errorCode = "1";
				}
			}
			if (!((RequiredDomValidator) Page.FindControl("DataValidator")).IsValid)
			{
				PageValid = false;
				errorCode += "2";
			}

			if (((TextBox) Page.FindControl("TextBoxSubject")).Text.Length <= 0)
			{
				PageValid = false;
				errorCode += "3";
			}

			if (PageValid &&
				(((TextBox) Page.FindControl("TextboxCompanyID")).Text.Length > 0 ||
				((TextBox) Page.FindControl("TextboxContactID")).Text.Length > 0 ||
				((TextBox) Page.FindControl("TextboxLeadID")).Text.Length > 0))
			{
				DigiDapter dg = new DigiDapter();

				dg.Add("Type", a, 'I');
				dg.Add("CreatedDate", DateTime.UtcNow, 'I');
				dg.Add("CreatedByID", UC.UserId, 'I');


				dg.Add("OwnerID", ((TextBox) Page.FindControl("TextboxOwnerID")).Text);


				if (((TextBox) Page.FindControl("TextboxCompanyID")).Text.Length > 0)
					dg.Add("CompanyID", ((TextBox) Page.FindControl("TextboxCompanyID")).Text);
				else
				{
					if(((TextBox) Page.FindControl("TextboxContactID")).Text.Length>0)
					{
						try
						{
							string cid = DatabaseConnection.SqlScalar("select companyid from base_contacts where id="+int.Parse(((TextBox) Page.FindControl("TextboxContactID")).Text));
							if(int.Parse(cid)>0)
							{
								dg.Add("CompanyID", cid);
							}
						}
						catch
						{
							dg.Add("CompanyID", DBNull.Value);
						}
					}
					else
						dg.Add("CompanyID", DBNull.Value);

				}

				if (((TextBox) Page.FindControl("TextboxContactID")).Text.Length > 0)
					dg.Add("ReferrerID", ((TextBox) Page.FindControl("TextboxContactID")).Text);
				else
					dg.Add("ReferrerID", DBNull.Value);

				if (((TextBox) Page.FindControl("TextboxLeadID")).Text.Length > 0)
					dg.Add("LeadID", ((TextBox) Page.FindControl("TextboxLeadID")).Text);
				else
					dg.Add("LeadID", DBNull.Value);

				if (((TextBox) Page.FindControl("TextboxOpportunityID")).Text.Length > 0)
					if (DatabaseConnection.SqlScalar("select count(id) from crm_Opportunity where id=" + int.Parse(((TextBox) Page.FindControl("TextboxOpportunityID")).Text)) != "0")
						dg.Add("OpportunityID", ((TextBox) Page.FindControl("TextboxOpportunityID")).Text);
					else
						dg.Add("OpportunityID", DBNull.Value);
				else
					dg.Add("OpportunityID", DBNull.Value);

				if (((TextBox) Page.FindControl("IDDocument")).Text.Length > 0)
					dg.Add("DocID", ((TextBox) Page.FindControl("IDDocument")).Text);
				else
					dg.Add("DocID", DBNull.Value);

				dg.Add("Subject", ((TextBox) Page.FindControl("TextBoxSubject")).Text);
				dg.Add("Description", ((TextBox) Page.FindControl("TextboxDescription")).Text);
				dg.Add("Description2", ((TextBox) Page.FindControl("TextboxDescription2")).Text);

				dg.Add("Groups", "|" + UC.UserGroupId.ToString() + "|");
				dg.Add("LastModifiedDate", DateTime.UtcNow);
				dg.Add("LastModifiedByID", UC.UserId.ToString());
				dg.Add("InOut", (((RadioButtonList) Page.FindControl("Activity_InOut")).SelectedValue == "1") ? true : false);
				dg.Add("ToDo", ((RadioButtonList) Page.FindControl("Activity_ToDo")).SelectedValue);

				if (((TextBox) Page.FindControl("F_StartHour")).Text.Length > 0)
					((TextBox) Page.FindControl("TextBoxHour")).Text = ((TextBox) Page.FindControl("F_StartHour")).Text;


				dg.Add("ActivityDate", UC.LTZ.ToUniversalTime(Convert.ToDateTime(((TextBox) Page.FindControl("TextBoxData")).Text + " " + ((TextBox) Page.FindControl("TextBoxHour")).Text, UC.myDTFI)));
				if (((DropDownList) Page.FindControl("DropDownListClassification")).SelectedIndex != -1)
					dg.Add("Classification", ((DropDownList) Page.FindControl("DropDownListClassification")).SelectedValue);
				if (((TextBox) Page.FindControl("TextboxParentID")).Text.Length > 0)
					dg.Add("ParentID", ((TextBox) Page.FindControl("TextboxParentID")).Text);
				else
					dg.Add("ParentID", 0);
				dg.Add("State", ((DropDownList) Page.FindControl("DropDownListStatus")).SelectedValue);
				dg.Add("Priority", ((DropDownList) Page.FindControl("DropDownListPriority")).SelectedValue);

				dg.Add("ToBill", ((CheckBox) Page.FindControl("CheckToBill")).Checked);
				dg.Add("Commercial", ((CheckBox) Page.FindControl("CheckCommercial")).Checked);
				dg.Add("Technical", ((CheckBox) Page.FindControl("CheckTechnical")).Checked);

				if (((DropDownList) Page.FindControl("DropDownListPreAlarm")).SelectedIndex > 0)
				{
					dg.Add("Allarm", UC.LTZ.ToUniversalTime(Convert.ToDateTime(((TextBox) Page.FindControl("TextBoxData")).Text, UC.myDTFI)));
					dg.Add("DaysAllarm", ((DropDownList) Page.FindControl("DropDownListPreAlarm")).SelectedValue);
				}

				int duration = 0;
				if (((TextBox) Page.FindControl("TextBoxDurationH")).Text.Length > 0)
				{
					try
					{
						duration += Convert.ToInt32(((TextBox) Page.FindControl("TextBoxDurationH")).Text)*60;
					}
					catch
					{
						duration += 0;
					}
				}

				if (((TextBox) Page.FindControl("TextBoxDurationM")).Text.Length > 0)
				{
					try
					{
						duration += Convert.ToInt32(((TextBox) Page.FindControl("TextBoxDurationM")).Text);
					}
					catch
					{
						duration += 0;
					}
				}
				dg.Add("Duration", duration);

				dg.Execute("CRM_WorkActivity", "ID=" + id, DigiDapter.Identities.Row);

				DataRow myDataRow = dg.GetNewRow;
				DataRow oldData = dg.GetOriginalRow;

				if (((CheckBox) Page.FindControl("CheckSendMail")).Checked && ((HtmlInputText) Page.FindControl("destinationEmail")).Value.Length > 0)
				{
					if (dg.RecordInserted)
						DatabaseConnection.DoCommand("INSERT INTO ActivityMoveLog (acId, ActionType, PrevValue, NextValue, MoveDate, OwnerID) VALUES   ('" + myDataRow["ID"].ToString() + "', " + (byte) ActivityMoveLog.MailSent + ", '" + UC.UserName + "', '" + DatabaseConnection.FilterInjection(((HtmlInputText) Page.FindControl("destinationEmail")).Value) + "', getdate(), '" + UC.UserId + "')");
					else
						DatabaseConnection.DoCommand("INSERT INTO ActivityMoveLog (acId, ActionType, PrevValue, NextValue, MoveDate, OwnerID) VALUES   ('" + id + "', " + (byte) ActivityMoveLog.MailSent + ", '" + UC.UserName + "', '" + DatabaseConnection.FilterInjection(((HtmlInputText) Page.FindControl("destinationEmail")).Value) + "', getdate(), '" + UC.UserId + "')");
				}

				((Label) Page.FindControl("ActivityID")).Text = myDataRow["ID"].ToString();
				int newId = Convert.ToInt32(myDataRow["ID"]);
				if (((RadioButtonList) Page.FindControl("Activity_ToDo")).SelectedValue == "0" || ((DropDownList) Page.FindControl("DropDownListPreAlarm")).SelectedValue.Length > 0)
					ModifyReminderActivity(newId, ((TextBox) Page.FindControl("TextBoxData")).Text);

				if (dg.RecordInserted)
				{
					int oId = int.Parse(((TextBox) Page.FindControl("TextboxOwnerID")).Text);
					if (((TextBox) Page.FindControl("TextboxCompanyID")).Text.Length > 0)
					{
						int compId = int.Parse(((TextBox) Page.FindControl("TextboxCompanyID")).Text);
						string acId = DatabaseConnection.SqlScalar("select id from LastContact where TableID=0 and Account=" + oId + " and CrossID=" + compId);
						if (acId.Length > 0)
							DatabaseConnection.DoCommand("update LastContact set ActivityID=" + newId + ",ActivityDate=getdate() where ID=" + acId);
						else
							DatabaseConnection.DoCommand("insert into LastContact (Account,ActivityID,CrossID,TableID) values (" + oId + "," + newId + "," + compId + ",0)");
					}
					if (((TextBox) Page.FindControl("TextboxContactID")).Text.Length > 0)
					{
						int contId = int.Parse(((TextBox) Page.FindControl("TextboxContactID")).Text);
						string acId = DatabaseConnection.SqlScalar("select id from LastContact where TableID=1 and Account=" + oId + " and CrossID=" + contId);
						if (acId.Length > 0)
							DatabaseConnection.DoCommand("update LastContact set ActivityID=" + newId + ",ActivityDate=getdate() where ID=" + acId);
						else
							DatabaseConnection.DoCommand("insert into LastContact (Account,ActivityID,CrossID,TableID) values (" + oId + "," + newId + "," + contId + ",1)");
					}
					if (((TextBox) Page.FindControl("TextboxLeadID")).Text.Length > 0)
					{
						int leadId = int.Parse(((TextBox) Page.FindControl("TextboxLeadID")).Text);
						string acId = DatabaseConnection.SqlScalar("select id from LastContact where TableID=2 and Account=" + oId + " and CrossID=" + leadId);
						if (acId.Length > 0)
							DatabaseConnection.DoCommand("update LastContact set ActivityID=" + newId + ",ActivityDate=getdate() where ID=" + acId);
						else
							DatabaseConnection.DoCommand("insert into LastContact (Account,ActivityID,CrossID,TableID) values (" + oId + "," + newId + "," + leadId + ",2)");
					}
				}

				if (((RadioButtonList) Page.FindControl("Activity_ToDo")).Items[0].Selected && a == "6")
					((HtmlContainerControl) Page.FindControl("Appointmenthours")).Attributes.Add("style", "display:none");


				if (dg.recordUpdated)
				{
					if (oldData["OwnerID"].ToString() != myDataRow["OwnerID"].ToString())
						DatabaseConnection.DoCommand("INSERT INTO ActivityMoveLog (acId, ActionType, PrevValue, NextValue, MoveDate, OwnerID) VALUES   (" + id + ", " + (byte) ActivityMoveLog.MoveOwner + ", '" + oldData["OwnerID"].ToString() + "', '" + myDataRow["OwnerID"].ToString() + "', getdate(), '" + UC.UserId + "')");

					if (oldData["ActivityDate"].ToString() != myDataRow["ActivityDate"].ToString())
							DatabaseConnection.DoCommand("INSERT INTO ActivityMoveLog (acId, ActionType, PrevValue, NextValue, MoveDate, OwnerID) VALUES   (" + id + ", " + (byte) ActivityMoveLog.MoveDate + ", '" + (UC.LTZ.ToLocalTime(Convert.ToDateTime( oldData["ActivityDate"]))).ToString(@"yyyyMMdd H.mm") + "', '" + (UC.LTZ.ToLocalTime(Convert.ToDateTime( myDataRow["ActivityDate"]))).ToString(@"yyyyMMdd H.mm") + "', getdate(), '" + UC.UserId + "')");

					if (oldData["CompanyID"].ToString() != myDataRow["CompanyID"].ToString())
						DatabaseConnection.DoCommand("INSERT INTO ActivityMoveLog (acId, ActionType, PrevValue, NextValue, MoveDate, OwnerID) VALUES   (" + id + ", " + (byte) ActivityMoveLog.MoveCompany + ", '" + oldData["CompanyID"].ToString() + "', '" + myDataRow["CompanyID"].ToString() + "', getdate(), '" + UC.UserId + "')");

					if (oldData["ReferrerID"].ToString() != myDataRow["ReferrerID"].ToString())
						DatabaseConnection.DoCommand("INSERT INTO ActivityMoveLog (acId, ActionType, PrevValue, NextValue, MoveDate, OwnerID) VALUES   (" + id + ", " + (byte) ActivityMoveLog.MoveContact + ", '" + oldData["ReferrerID"].ToString() + "', '" + myDataRow["ReferrerID"].ToString() + "', getdate(), '" + UC.UserId + "')");

					if (oldData["LeadID"].ToString() != myDataRow["LeadID"].ToString())
						DatabaseConnection.DoCommand("INSERT INTO ActivityMoveLog (acId, ActionType, PrevValue, NextValue, MoveDate, OwnerID) VALUES   (" + id + ", " + (byte) ActivityMoveLog.MoveLead + ", '" + oldData["LeadID"].ToString() + "', '" + myDataRow["LeadID"].ToString() + "', getdate(), '" + UC.UserId + "')");

					if (oldData["ToDo"].ToString() != myDataRow["ToDo"].ToString())
						DatabaseConnection.DoCommand("INSERT INTO ActivityMoveLog (acId, ActionType, PrevValue, NextValue, MoveDate, OwnerID) VALUES   (" + id + ", " + (byte) ActivityMoveLog.MoveDone + ", '" + oldData["ToDo"].ToString() + "', '" + myDataRow["ToDo"].ToString() + "', getdate(), '" + UC.UserId + "')");
				}

				return "0";
			}
			else
			{
				string error = String.Empty;
				if (errorCode.IndexOf("1") > -1 || errorCode.IndexOf("2") > -1)
					error += rm.GetString("Acttxt101") + "<br>";

				if (!(((TextBox) Page.FindControl("TextBoxSubject")).Text.Length > -1))
					error += rm.GetString("Acttxt100") + "<br>";

				if (!(((TextBox) Page.FindControl("TextboxCompanyID")).Text.Length > -1) &&
					!(((TextBox) Page.FindControl("TextboxContactID")).Text.Length > -1))
					error += rm.GetString("Acttxt102") + "<br>";

				if (((RadioButtonList) Page.FindControl("Activity_ToDo")).Items[0].Selected && a == "6")
					((HtmlContainerControl) Page.FindControl("Appointmenthours")).Attributes.Add("style", "display:none");

				return error;
			}
		}
		}
	}
}
